﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace ParticleEffectsDemo
{
    public partial class Page : UserControl
    {
        private const double fadeSpeed = 0.015D;
        private const double floatUpSpeed = 2D;
        private const int driftDelay = 1000;
        private DispatcherTimer timerLoop;
        private ParticleGenerator<SmokeParticle> particles = new ParticleGenerator<SmokeParticle>();

        public Page()
        {
            InitializeComponent();
            timerLoop = new DispatcherTimer();
            timerLoop.Interval = TimeSpan.FromMilliseconds(50);
            timerLoop.Tick += new EventHandler(timerLoop_Tick);
            timerLoop.Start();
        }

        void timerLoop_Tick(object sender, EventArgs e)
        {
            ScaleTransform st;
            CreateParticle(new Point(this.Width / 2, this.Height - 20));

            // Update position and opacity of all existing particles
            foreach (SmokeParticle p in LayoutRoot.Children.OfType<SmokeParticle>().ToList())
            {
                st = p.RenderTransform as ScaleTransform;
                if (st != null)
                {
                    st.ScaleX += fadeSpeed;
                    st.ScaleY += fadeSpeed;
                    p.Opacity -= fadeSpeed;
                    double y = (double)p.GetValue(Canvas.TopProperty);
                    p.SetValue(Canvas.TopProperty, y - floatUpSpeed);
                    if (st.ScaleX >= 1)
                    {
                        LayoutRoot.Children.Remove(p);
                        p.Drift.Stop();
                        particles.ReturnParticle(p);
                    }
                }
            }
        }

        private void CreateParticle(Point pt)
        {
            // Create a new particle
            SmokeParticle p;
            ScaleTransform st;
            p = particles.GetParticle();

            // Scale it to the correct size
            st = new ScaleTransform();
            st.CenterX = 20;
            st.CenterY = 20;
            st.ScaleX = 0.1;
            st.ScaleY = 0.1;
            p.RenderTransform = st;

            // Set opacity to visible
            p.Opacity = 1;

            // Set the particle start position with a slight random offset
            p.SetValue(Canvas.LeftProperty, new Random().Next(5) + pt.X);
            p.SetValue(Canvas.TopProperty, pt.Y);

            // Start the drift animation on the particle
            p.Drift.BeginTime = TimeSpan.FromMilliseconds(new Random().Next(driftDelay));
            p.Drift.Begin();

            // Add to the canvas
            LayoutRoot.Children.Add(p);
        }


    }
}
